import networkx as nx
from fixed_node import *

##
## Fixed Binary Tree class
##
class FixedBTree:
    
    ## data structures
    T = None        # top-down tree
    Tinv = None     # bottom-up tree
    root = None     # root of the tree
    Depth = None    # depth of a tree
    nTerm = None    # # of terminal nodes
    
    nodes = None
    last_idx = -1
    
    ## methods
    def __init__(self):
        """
        FixedBTree()        
        """
        pass
    
    def init_bottom_up(self, maxdepth):
        T = nx.DiGraph()
        Tinv = nx.DiGraph()
        nodes = []
        
        NN = 2**maxdepth
        
        # the very bottom nodes
        idx = self.last_idx
        for ii in xrange(NN):
            idx += 1
            inode = InternNode(idx)
            inode.prefix = ii
            inode.depth = maxdepth
            
            nodes.append(inode)
        
        # Bottom-up
        Q = [nn for nn in nodes]
        while len(Q) > 1 :
            v1 = Q.pop(0)
            v2 = Q.pop(0)
            
            # Should be so, because of a full binary tree
            assert( v1.depth == v2.depth )
            idx += 1
            vint = InternNode(idx)
            
            vint.depth = v1.depth - 1
            vint.prefix = merged_prefix(v1,v2)
            
            # Grow tree topology
            T.add_edge( vint, v1 )
            T.add_edge( vint, v2 )
            Tinv.add_edge( v1, vint )
            Tinv.add_edge( v2, vint )
            
            Q.append(vint)
            nodes.append(vint)
            pass
        
        # save the tree
        self.root = Q.pop(0)        
        self.T = T
        self.Tinv = Tinv
        self.Depth = maxdepth        
        self.nodes = nodes
        self.last_idx = idx
        
        return 0
    
    def init_params(self,nparam,node_type=None):
        for nn in self.nodes:
            intern_init_param(nn,nparam,node_type)
        return 0
    
    def leaves(self):
        return self.nodes[:2**self.Depth]
    
    def nonleaves(self):
        return self.nodes[2**self.Depth:]
        
    def parent(self,r):
        return self.Tinv.neighbors(r)[0]
    
    def sibling(self,r):        
        tmp = [nn for nn in self.T.neighbors(self.parent(r))
               if nn != r]
        return tmp[0]
    
    def children(self,r):
        return self.T.neighbors(r)
    
    def dfs_traversal(self):
        return nx.search.dfs_postorder(self.T, self.root)
    
    ## terminal accessing methods    
    def left_terminals_under(self,r):
        """
        xrange_list = left_terminals(self,r):
        @param r: internal node r
        @return: a list of terminals (indexes)
                that are under the left child of 'r'
        """
        if r.depth == self.Depth : 
            raise RuntimeError('Invalid at the bottom')
        return intern_find_left(r, self.Depth)
    
    def right_terminals_under(self,r):
        """
        xrange_list = right_terminals(self,r):
        @param r: internal node r
        @return: a list of terminals (indexes)
                that are under the right child of 'r'
        """
        if r.depth == self.Depth : 
            raise RuntimeError('Invalid at the bottom')
        return intern_find_right(r, self.Depth)
    
    def terminals_under_sibling(self,r):        
        sib = self.sibling(r)
        return self.terminals_under(sib)
    
    def terminals_under(self,r):
        """
        xrange_list = terminals(self,r):
        @param r: internal node r
        @return: a list of terminals (indexes) 
                that are under the internal 'r'
        """
        if r.depth == self.Depth:
            return [r.prefix]
        
        llist = intern_find_left(r, self.Depth)
        rlist = intern_find_right(r, self.Depth)
        
        return xrange(llist[0],rlist[-1]+1)
    
    pass #------- End of class fixed perfect binary tree ------#

