'''
Created on Aug 5, 2009
Modified on Oct 8, 2009

@author: yongjin
'''
import sys; import os;
sys.path.append( os.path.abspath('.') )

from numpy import zeros, mean, std, dot, eye, diag, argmax, inf, logical_and
from inference.hrg import TimeVaryingEngine
from inference.linkpred import linkupdate
from scipy.io import savemat
import networkx as nx


###################
def run_exp(tengine,lmd,trials):
    
    # multiple trials to avoid getting stuck in local optima
    max_latent = None; max_llik = -inf    
    for trial in xrange(trials):
        tengine.learn_stoc(10, 9, lmd=lmd, eta=1e-2, warmstart=False)
        llik = tengine.learn_mean(100, lmd=lmd, tol=5e-2, warmstart=True)
        # llik = tengine.learn_max(100, lmd=lmd, eta=1e-2, tol=5e-2, warmstart=True)
        if llik[-1] > max_llik:
            max_llik = llik[-1]
            max_latent = tengine.export_latent_matrix()
            pass
        pass
    
    # set to max latent variables
    tengine.import_latent_matrix(max_latent)
    
    # update tree parameters according to argmax  
    for tt in xrange(tengine.T): tengine.update_tree_mf(tt)
    # update link prediction
    tengine.init_link()
    linkupdate(tengine, cont=True)
    Link = tengine.export_link()
    
    return max_latent, Link

####################


if __name__ == '__main__':
    
    ####### Input of DYHM
    #
    # Read in NX graph into a dictionary (e.g., at time '0' we have graph G)
    dataGraphs = {}    
    for ii in xrange(10) : 
        dataGraphs['%d'%ii] = nx.read_adjlist( '../data/G%d.adjlist.gz'%ii )
        pass
    
    # time dependency graph    
    timeG = nx.read_adjlist( '../data/T.adjlist.gz' )
        
    # keep one list of vertices to preserve orders
    fh = open( '../data/vertices.txt' )
    vertices = [ll.strip() for ll in fh.readlines()]
    fh.close()
    #
    #####
    
    #####
    # inference parameters
    #
    depth = 3;  # depth of tree structure
    lmd = 0.1;  # regularization parameter lambda
    trials = 5; # number of random re-starts
    
    # time-varying Hierarchical Random Graph model
    tengine = TimeVaryingEngine(dataGraphs,timeG,depth,vertices=vertices)    
    
    # do experiment with multiple random re-starts
    latent, link = run_exp(tengine,lmd,trials)
        
    #### save latent assignment and link prediction
    savemat("../result/latent.mat",latent,appendmat=False)
    savemat("../result/link.mat",link,appendmat=False)
        
    pass


